#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
import logging
from .cal_crawler import CalCrawler
from .healthy_checker import HealthyChecker
from .common_utils import CommonUtils


class AIMonitor(object):
    @staticmethod
    def monitor_live_box(host_name, start_time, end_time, pool_name="risktxncomputeserv"):
        cal_log_data = CalCrawler.crawl_data(host_name, start_time, end_time, pool_name=pool_name)
        check_result = HealthyChecker.abnormal_check(cal_log_data)
        healthy_status, detail_info = HealthyChecker.result_analysis(check_result)
        logging.debug("Check box %s on date %s to %s, CHECK RESULT: %s\n\n" % (host_name, CommonUtils.timestamp_to_data_str(start_time), CommonUtils.timestamp_to_data_str(end_time), detail_info))
        return healthy_status, detail_info

    @staticmethod
    def monitor_last_x_hour(host_name, hours=24, pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        last_x_hours_timestamp = current_timestamp - hours * 60 * 60
        return AIMonitor.monitor_live_box(host_name, last_x_hours_timestamp, current_timestamp, pool_name)

    @staticmethod
    def monitor_last_x_day(host_name, days=7, pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        last_x_days_timestamp = current_timestamp - days * 24 * 60 * 60
        return AIMonitor.monitor_live_box(host_name, last_x_days_timestamp, current_timestamp, pool_name)

    @staticmethod
    def monitor_one_box(one_box_list, one_box_start_time, time_format="%Y-%m-%d %H:%M:%S", pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        start_timestamp = CommonUtils.date_str_to_timestamp(one_box_start_time, time_format)
        one_box_result = {}
        for host_name in one_box_list:
            one_box_result[host_name] = AIMonitor.monitor_live_box(host_name, start_timestamp, current_timestamp,
                                                                   pool_name)
        return one_box_result


if __name__ == "__main__":
    AIMonitor.monitor_one_box(["dcg12risktxncomputeserv9421"], "2018-07-11 01:22:33", pool_name="risktxncomputeserv")





#!/usr/bin/python
# -*- coding: utf-8 -*-
from threading import Thread
from .common_utils import CommonUtils
from .cal_crawler import CalCrawler
from .ai_monitor import AIMonitor
from .healthy_checker import CheckStatus


THREAD_TIMEOUT = 300


class DailyChecker(object):
    @staticmethod
    def __healthy_check(host_name, start_time, end_time, check_date, check_result, pool_name="risktxncomputeserv"):
        check_result[check_date] = AIMonitor.monitor_live_box(host_name, start_time, end_time, pool_name)[0]
        print("Check %s on %s result: %s" % (host_name, check_date, check_result[check_date]))

    @staticmethod
    def check_for_host_current_month(host_name, callback_result):
        thread_list = []
        check_result = {}
        check_date_dict = CommonUtils.get_current_month_each_day_start_end_time()
        for check_date, time_tuple in check_date_dict.items():
            thread = Thread(target=DailyChecker.__healthy_check, args=[host_name, time_tuple[0], time_tuple[1],
                                                                       check_date, check_result])
            thread.daemon = True
            thread.start()
            thread_list.append(thread)
        for thread in thread_list:
            thread.join(THREAD_TIMEOUT)

        status_dict = {check_status: 0 for check_status in CheckStatus}
        for check_date, healthy_status in check_result.items():
            status_dict[healthy_status] += 1
        callback_result[host_name] = status_dict
        return callback_result

    @staticmethod
    def pool_check(pool_name="risktxncomputeserv"):
        thread_list = []
        check_result = {}
        for host_name in CalCrawler.get_all_traffic_host(pool_name):
            thread = Thread(target=DailyChecker.check_for_host_current_month, args=[host_name, check_result])
            thread.daemon = True
            thread.start()
            thread_list.append(thread)
        for thread in thread_list:
            thread.join(THREAD_TIMEOUT)
        return check_result

    @staticmethod
    def get_top_10_bad_box(pool_name="risktxncomputeserv"):
        check_result = DailyChecker.pool_check(pool_name)
        check_result_list = [(host_name, status_dict) for host_name, status_dict in check_result.items()]
        sorted_result = sorted(check_result_list, cmp=lambda x, y: cmp(x[1][CheckStatus.RED], y[1][CheckStatus.RED]),
                               reverse=True)
        return sorted_result[:10]


if __name__ == '__main__':
    # print(DailyChecker.pool_check())
    print(DailyChecker.check_for_host_current_month("ccg01risktxncomputeserv4184", {}))





#!/usr/bin/python
# -*- coding: utf-8 -*-
from enum import Enum
from string import Template


class DataType(Enum):
    REQUEST_COUNT = 1
    ERROR_COUNT = 2
    CAL_STATUS_1 = 3
    CAL_STATUS_2 = 4
    REQUEST_DURATION = 5
    CPU = 6
    MEMORY_USAGE = 7
    JVM = 8
    GC_COUNT = 9


def request_count_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appappmon.requestCount.STATE.60s%7Bpool%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def error_count_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appappmon.requestCount.STATE.60s%7Bpool%3D$pool_name%2Cstatus%3D1%7C2%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def cal_status_1_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appappmon.requestCount.STATE.60s%7Bpool%3D$pool_name%2Cstatus%3D1%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def cal_status_2_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appappmon.requestCount.STATE.60s%7Bpool%3D$pool_name%2Cstatus%3D2%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def request_duration_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appappmon.requestDuration.STATE.60s%7Bpool%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def cpu_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=avg%3Appsysmon.appcpu.usedperc.STATE.60s%7Bapp%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def memory_usage_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=avg%3Appsysmon.appmemr.used_perc.STATE.60s%7Bapp%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def jvm_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=avg%3Appsysmon.jvmm.heapfree_kb.STATE.60s%7Bapp%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


def gc_count_template():
    return Template("https://engineering.paypalcorp.com/sherlock/pp.tsdb/q?start=$start_time&end=$end_time&m=sum%3Appsysmon.jvmm.gccountPerSec.STATE.60s%7Bapp%3D$pool_name%2Ccolo%3D$column_name%2Chost%3D$host_name%7D&o=&format=$output_type&useTopoCache=$use_cache")


switcher = {
    DataType.REQUEST_COUNT: request_count_template(),
    DataType.ERROR_COUNT: error_count_template(),
    DataType.CAL_STATUS_1: cal_status_1_template(),
    DataType.CAL_STATUS_2: cal_status_2_template(),
    DataType.REQUEST_DURATION: request_duration_template(),
    DataType.CPU: cpu_template(),
    DataType.MEMORY_USAGE: memory_usage_template(),
    DataType.JVM: jvm_template(),
    DataType.GC_COUNT: gc_count_template()
}


class UrlTemplate(object):
    @staticmethod
    def get_template_by_data_type(type):
        return switcher.get(type)

    @staticmethod
    def render(data_type, start_time, end_time, pool_name, column_name, host_name, output_type, use_cache):
        template = UrlTemplate.get_template_by_data_type(data_type)
        if template is not None:
            return template.substitute(start_time = start_time, end_time = end_time, pool_name = pool_name, column_name = column_name, host_name = host_name, output_type = output_type, use_cache = str(use_cache).lower())
        print("Get template return None, Type %s is not one of data_template.data_type" % data_type.name)
        return None


if __name__ == '__main__':
    for _type in DataType:
        print(_type.name, UrlTemplate.render(_type, 1530082680, 1530086520, "risktxncomputeserv", "dcg13", "dcg13risktxncomputeserv7498", "json", True))



#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
import logging
import pytz
from datetime import datetime, date


class CommonUtils(object):
    @staticmethod
    def get_current_pst_timestamp():
        utc_date = datetime.now(tz=pytz.utc)
        pst_date = utc_date.astimezone(pytz.timezone('US/Pacific'))
        return int(time.mktime(pst_date.timetuple()))

    @staticmethod
    def date_str_to_timestamp(date_str, format="%Y-%m-%d %H:%M:%S"):
        return int(time.mktime(datetime.strptime(date_str, format).timetuple()))

    @staticmethod
    def timestamp_to_data_str(ts, format="%Y-%m-%d %H:%M:%S"):
        current_date = datetime.fromtimestamp(ts)
        return current_date.strftime(format)

    @staticmethod
    def get_column_name_from_host_name(host_name):
        dcg_list = ["dcg01", "dcg02", "dcg11", "dcg12", "dcg13"]
        for dcg in dcg_list:
            if host_name.startswith(dcg):
                return dcg
        if host_name.startswith("slc"):
            if host_name.endswith("a"):
                return "slca"
            elif host_name.endswith("b"):
                return "slcb"
        logging.warn("host %s can not be identified, class it into phx" % host_name)
        return "phx"

    @staticmethod
    def get_current_month_each_day_start_end_time():
        start_end_time_dict = {}
        today = datetime.today()
        last_timestamp = int(time.time())
        for day in range(today.day, 0, -1):
            xth_day = date(today.year, today.month, day)
            xth_day_start = int(time.mktime(xth_day.timetuple()))
            start_end_time_dict[xth_day] = (xth_day_start, last_timestamp)
            last_timestamp = xth_day_start - 1
        return start_end_time_dict

    @staticmethod
    def remove_serial_duplicate(duplicate_list):
        if len(duplicate_list) <= 1:
            return duplicate_list
        else:
            uniq_list = [duplicate_list[0]]
            for i in range(1, len(duplicate_list)):
                if duplicate_list[i][0] != uniq_list[-1][0]:
                    uniq_list.append(duplicate_list[i])
        return uniq_list


if __name__ == '__main__':
    print("Current time: %s" % CommonUtils.timestamp_to_data_str(int(time.time())))
    print("Current PST time: %s" % CommonUtils.timestamp_to_data_str(CommonUtils.get_current_pst_timestamp()))
    __host_name = "dcg13risktxncomputeserv6056"
    print("Pool name for %s is %s" % (__host_name, CommonUtils.get_column_name_from_host_name(__host_name)))





name = "cal_ai"

from .ai_monitor import AIMonitor
from .cal_crawler import CalCrawler, MachineInfo
from .cal_plot import CalPlot
from .common_utils import CommonUtils
from .data_template import DataType, UrlTemplate
from .healthy_checker import HealthyChecker



#!/usr/bin/python
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import datetime
import time
from .cal_crawler import CalCrawler
from .common_utils import CommonUtils
from .healthy_checker import MachineInfo


class CalPlot(object):
    @staticmethod
    def __plot(history_data):
        fig = plt.figure(figsize=(25, 25))
        sub_fig_num = len(history_data)
        base_value = sub_fig_num * 100 + 10 + sub_fig_num
        display_order = [MachineInfo.TPM, MachineInfo.TRANS_TIME, MachineInfo.CPU_USAGE, MachineInfo.MEMORY_USAGE,
                         MachineInfo.EPM, MachineInfo.JVM, MachineInfo.GC_COUNT, MachineInfo.CAL_STATUS_1,
                         MachineInfo.CAL_STATUS_2]
        for machine_info_type in display_order[::-1]:
            if machine_info_type in history_data:
                data_list = history_data[machine_info_type]
                ax = fig.add_subplot(base_value)
                base_value -= 1
                x, y = zip(*data_list)
                x = [datetime.datetime.fromtimestamp(ts) for ts in x]
                ax.plot(x, y)
                ax.set_title(machine_info_type.name)
                plt.subplots_adjust(hspace=0.3)
                plt.autoscale()
        plt.gca().xaxis.set_major_formatter(mdates.DateFormatter("%Y-%m-%d %H:%M"))
        plt.gca().xaxis.set_major_locator(mdates.HourLocator())
        plt.gcf().autofmt_xdate()
        plt.show()

    @staticmethod
    def show_instance_history(host_name, start_time_str, end_time_str=None, time_str_format="%Y-%m-%d %H:%M:%S"):
        if end_time_str is None:
            end_time = int(time.time())
        else:
            end_time = CommonUtils.date_str_to_timestamp(end_time_str, time_str_format)
        start_time = CommonUtils.date_str_to_timestamp(start_time_str, time_str_format)

        result = CalCrawler.crawl_data(host_name, start_time, end_time)
        history_data = {}
        for data_type, data_list in result.items():
            if len(data_list) > 0 and 'DataPoints' in data_list[0] and len(data_list[0]['DataPoints']) > 0:
                history_data[data_type] = data_list[0]['DataPoints']
        CalPlot.__plot(history_data)

    @staticmethod
    def monitor_last_x_hours(host_name, last_x_hours, pool_name="risktxncomputeserv"):
        end_time = int(time.time())
        start_time = end_time - last_x_hours * 3600
        result = CalCrawler.crawl_data(host_name, start_time, end_time, pool_name=pool_name)
        history_data = {}
        for data_type, data_list in result.items():
            if len(data_list) > 0 and 'DataPoints' in data_list[0] and len(data_list[0]['DataPoints']) > 0:
                history_data[data_type] = data_list[0]['DataPoints']
        CalPlot.__plot(history_data)


if __name__ == "__main__":
    CalPlot.show_instance_history("dcg12risktxncomputeserv6950", "2018-07-09 10:12:12", "2018-07-10 10:12:12")






#!/usr/bin/python
# -*- coding: utf-8 -*-
import logging
from enum import Enum
from string import Template
from .common_utils import CommonUtils
from .cal_crawler import MachineInfo


abnormal_wave_template = Template("Abnormal high $machine_info: From $start_time to $end_time, Max $max_value at $max_time")

healthy_threshold = {
    MachineInfo.TPM: 500,
    MachineInfo.EPM: 2,
    MachineInfo.CAL_STATUS_1: 2,
    MachineInfo.CAL_STATUS_2: 2,
    MachineInfo.TRANS_TIME: 800,
    MachineInfo.CPU_USAGE: 50,
    MachineInfo.MEMORY_USAGE: 55,
    MachineInfo.JVM: 1,
    MachineInfo.GC_COUNT: 1
}


class CheckStatus(Enum):
    Green = 0
    YELLOW = 1
    RED = 2


class HealthyChecker(object):
    @staticmethod
    def abnormal_check(machine_info_status):
        return {machine_info_type: HealthyChecker.__basic_abnormal_check(machine_info_type, data_list) for machine_info_type, data_list in machine_info_status.items()}

    @staticmethod
    def result_analysis(abnormal_result):
        assessments = []
        for machine_info in MachineInfo:
            if machine_info in abnormal_result:
                abnormal_behaviours = abnormal_result.get(machine_info)
                for abnormal_behaviour in abnormal_behaviours:
                    assessments.append(abnormal_wave_template.substitute(machine_info=machine_info.name,
                                                                         start_time=CommonUtils.timestamp_to_data_str(abnormal_behaviour[0]),
                                                                         end_time=CommonUtils.timestamp_to_data_str(abnormal_behaviour[1]),
                                                                         max_time=CommonUtils.timestamp_to_data_str(abnormal_behaviour[2]),
                                                                         max_value=abnormal_behaviour[3]))
        if len(assessments) == 0:
            return CheckStatus.Green, "Healthy!"
        elif MachineInfo.TPM in abnormal_result:
            return CheckStatus.RED, "Abnormal behaviour identified.\nDetails:\n\t" + "\n\t".join(assessments)
        else:
            return CheckStatus.YELLOW, "Abnormal behaviour identified.\nDetails:\n\t" + "\n\t".join(assessments)

    @staticmethod
    def __abnormal_wave_detect(machine_info_type, data_list, merge_wave_interval=1800):
        abnormal_wave_list = []
        start_time = end_time = max_time = 0
        max_value = -1
        for timestamp, count in data_list:
            if count > healthy_threshold.get(machine_info_type):
                if start_time == 0:
                    start_time = timestamp
                if count > max_value:
                    max_value = count
                    max_time = timestamp
            else:
                if start_time != 0:
                    end_time = timestamp
                    wave = (start_time, end_time, max_time, max_value)
                    HealthyChecker.__append_merge(abnormal_wave_list, wave, merge_wave_interval)
                    start_time = 0
                    max_value = -1

        return abnormal_wave_list

    @staticmethod
    def __append_merge(full_list, element, merge_wave_interval):
        if len(full_list) == 0:
            full_list.append(element)
        else:
            last_element = full_list[-1]
            if element[0] - last_element[1] < merge_wave_interval:
                max_time, max_value = (element[2], element[3]) if element[3] > last_element[3] else (last_element[2], last_element[3])
                new_element = (last_element[0], element[1], max_time, max_value)
                full_list[-1] = new_element
            else:
                full_list.append(element)

    @staticmethod
    def __basic_abnormal_check(machine_info_type, data_list):
        logging.debug("Check machine info %s, data list:%s" % (machine_info_type, data_list))
        if data_list is None or len(data_list) == 0:
            logging.debug("Data list is null or empty for type: %s" % (machine_info_type))
            return []
        data_count_list = data_list[0]["DataPoints"]
        return HealthyChecker.__abnormal_wave_detect(machine_info_type, data_count_list)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
import logging
from enum import Enum
import requests, json
from threading import Thread
from .data_template import UrlTemplate, DataType
from .common_utils import CommonUtils


class MachineInfo(Enum):
    TPM = 1
    EPM = 2
    CAL_STATUS_1 = 3
    CAL_STATUS_2 = 4
    TRANS_TIME = 5
    CPU_USAGE = 6
    MEMORY_USAGE = 7
    JVM = 8
    GC_COUNT = 9


class CalCrawler(object):
    @staticmethod
    def get_all_traffic_host(pool_name="risktxncomputeserv", sample_width=3600, sample_count=5, output_type="json", use_cache=True,
                             max_wait_second=300):
        thread_list = []
        host_list = []
        current_timestamp = int(time.time())
        for i in range(sample_count):
            data_url = UrlTemplate.render(DataType.REQUEST_COUNT, current_timestamp - sample_width * i - 60,
                                          current_timestamp - sample_width * i, pool_name, "*", "*",
                                          output_type, use_cache)
            thread = Thread(target=CalCrawler.__crawl_hostname, args=[data_url, host_list])
            thread.daemon = True
            thread.start()
            thread_list.append(thread)

        for thread in thread_list:
            thread.join(max_wait_second)

        return list(set(host_list))

    @staticmethod
    def __crawl_hostname(data_url, host_list, retry=5):
        for i in range(retry):
            try:
                response = requests.get(data_url)
                data = json.loads(response.content)
                try:
                    host_list.extend([record["Tags"]["host"] for record in data])
                except KeyError:
                    print("Key error load %s" % data_url)
                    print("data %s" % data)
                if i > 0:
                    logging.warn("Success get data from url %s through %d retry when IOException." % (str(data_url), i))
                break
            except IOError:
                # Retry when IO Error Happen
                if i == retry - 1:
                    logging.error("Curl data failed after %d time retry, url %s" % (i, data_url))

    @staticmethod
    def __crawl(data_url, data_type, current_part_num, history_data, retry=5):
        start_time = time.time()
        logging.debug("%s thread start at time: %s" % (data_type, CommonUtils.timestamp_to_data_str(start_time)))
        logging.debug(data_url)

        for i in range(retry):
            try:
                response = requests.get(data_url)
                data = json.loads(response.content)
                history_data[data_type][current_part_num] = data
                if i > 0:
                    logging.warn("Success get data for data type %s through %d retry when IOException." % (str(data_type), i))
                break
            except IOError:
                # Retry when IO Error Happen
                if i == retry - 1:
                    logging.error("Curl data type %s failed after %d time retry, url %s" % (data_type, i, data_url))
            time.sleep(3)

        finish_time = time.time()
        logging.debug("%s thread finished at time: %s, total take %s second" % (data_type, CommonUtils.timestamp_to_data_str(finish_time), str(finish_time - start_time)))

    @staticmethod
    def __merge_history_data(history_data):
        merged_history_data = {}
        for data_type in DataType:
            data_type_dict = history_data[data_type]
            if not data_type_dict:
                logging.warn("Data crawl for type %s is None" % data_type.name)
            elif len(data_type_dict) != max(data_type_dict.keys()):
                logging.error("Crawl data part missing for type %s, collected parts: %s" % (data_type.name, str(data_type.keys())))
            else:
                merged_history_data[data_type] = []
                for i in range(1, len(data_type_dict) + 1):
                    logging.info("start merge %d for type %s" % (i, data_type.name))
                    if data_type_dict[i]:
                        if not merged_history_data[data_type]:
                            merged_history_data[data_type] = data_type_dict[i]
                        else:
                            merged_history_data[data_type][0]['DataPoints'].extend(data_type_dict[i][0]['DataPoints'])
        return merged_history_data

    @staticmethod
    def crawl_data(host_name, start_time, end_time, column_name=None, pool_name="risktxncomputeserv", output_type="json"
                   , use_cache=True, max_wait_second=300):
        if column_name is None:
            column_name = CommonUtils.get_column_name_from_host_name(host_name)
            logging.debug("Get column name %s from host %s" % (column_name, host_name))

        thread_list = []
        history_data = {}
        for data_type in DataType:
            history_data[data_type] = {}
            current_part_num = 1
            current_time = start_time
            while current_time + 24 * 60 * 60 < end_time:
                CalCrawler.__new_crawl_thread(data_type, current_time, current_time + 24 * 60 * 60, pool_name,
                                              column_name, host_name, output_type, use_cache, current_part_num,
                                              thread_list, history_data)
                current_time += 24 * 60 * 60
                current_part_num += 1
            CalCrawler.__new_crawl_thread(data_type, current_time, end_time, pool_name, column_name, host_name,
                                          output_type, use_cache, current_part_num, thread_list, history_data)
        for thread in thread_list:
            thread.join(max_wait_second)

        return CalCrawler.__convert_to_machine_info_history(CalCrawler.__merge_history_data(history_data))

    @staticmethod
    def __new_crawl_thread(data_type, start_time, end_time, pool_name, column_name, host_name, output_type, use_cache,
                           current_part_num, thread_list, history_data):
        data_url = UrlTemplate.render(data_type, start_time, end_time, pool_name, column_name, host_name, output_type, use_cache)
        thread = Thread(target=CalCrawler.__crawl, args=[data_url, data_type, current_part_num, history_data])
        thread.daemon = True
        thread.start()
        thread_list.append(thread)

    @staticmethod
    def __convert_to_machine_info_history(history_data):
        machine_status = {}
        if DataType.REQUEST_COUNT in history_data:
            machine_status[MachineInfo.TPM] = history_data[DataType.REQUEST_COUNT]
        if DataType.REQUEST_COUNT in history_data and DataType.REQUEST_DURATION in history_data:
            request_count_list = history_data.get(DataType.REQUEST_COUNT)
            duration_data_list = history_data.get(DataType.REQUEST_DURATION)
            if len(request_count_list) > 0 and len(duration_data_list) > 0:
                request_count_list = request_count_list[0]["DataPoints"]
                duration_data_list = duration_data_list[0]["DataPoints"]
                time_union = set([timestamp for timestamp, count in request_count_list]).intersection(set([ts for ts, duration_count in duration_data_list]))
                if not time_union:
                    logging.error("Empty union timestamp between request count list %d and duration data list %d" % (len(request_count_list), len(duration_data_list)))
                else:
                    request_count_list = [tc_tuple for tc_tuple in request_count_list if tc_tuple[0] in time_union]
                    duration_data_list = [tc_tuple for tc_tuple in duration_data_list if tc_tuple[0] in time_union]
                    if len(request_count_list) == len(duration_data_list):
                        machine_status[MachineInfo.TRANS_TIME] = history_data.get(DataType.REQUEST_DURATION)
                        machine_status[MachineInfo.TRANS_TIME][0]['DataPoints'] = [[request_count_list[i][0], duration_data_list[i][1] / request_count_list[i][1]] for i in range(len(request_count_list))]
                    else:
                        logging.error("request count list count %d not equal to duration data list count %d" % (len(request_count_list), len(duration_data_list)))
            else:
                logging.warn("Either request count %d or duration count %d is None." % (len(request_count_list), len(duration_data_list)))
        if DataType.ERROR_COUNT in history_data:
            machine_status[MachineInfo.EPM] = history_data[DataType.ERROR_COUNT]
        if DataType.CAL_STATUS_1 in history_data:
            machine_status[MachineInfo.CAL_STATUS_1] = history_data[DataType.CAL_STATUS_1]
        if DataType.CAL_STATUS_2 in history_data:
            machine_status[MachineInfo.CAL_STATUS_2] = history_data[DataType.CAL_STATUS_2]
        if DataType.CPU in history_data:
            machine_status[MachineInfo.CPU_USAGE] = history_data[DataType.CPU]
        if DataType.MEMORY_USAGE in history_data:
            machine_status[MachineInfo.MEMORY_USAGE] = history_data[DataType.MEMORY_USAGE]
        if DataType.ERROR_COUNT in history_data:
            machine_status[MachineInfo.EPM] = history_data[DataType.ERROR_COUNT]
        if DataType.JVM in history_data:
            machine_status[MachineInfo.JVM] = history_data[DataType.JVM]
        if DataType.GC_COUNT in history_data:
            machine_status[MachineInfo.GC_COUNT] = history_data[DataType.GC_COUNT]
        return machine_status


if __name__ == "__main__":
    print(len(CalCrawler.get_all_traffic_host("risktxncomputeserv")))
    result = CalCrawler.crawl_data("dcg13risktxncomputeserv6056", 1530979200, 1531324800)
    for key, value in result.items():
        print(key, value)


